# Generated by Django 2.2.2 on 2019-07-05 23:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_visit_schedule", "0004_auto_20190629_1800")]

    operations = [
        migrations.AlterField(
            model_name="historicalvisitschedule",
            name="timepoint",
            field=models.DecimalField(decimal_places=1, max_digits=6, null=True),
        ),
        migrations.AlterField(
            model_name="visitschedule",
            name="timepoint",
            field=models.DecimalField(decimal_places=1, max_digits=6, null=True),
        ),
    ]
